/*
 * Provide Test Panel for base XMPP actions and related tests
 * By SANKA.ir
 */

package ir.sanka.androiddev;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;


import org.jivesoftware.smack.Connection;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.provider.ProviderManager;
import org.jivesoftware.smack.util.StringUtils;
import org.jivesoftware.smackx.bytestreams.socks5.provider.BytestreamsProvider;
import org.jivesoftware.smackx.filetransfer.FileTransferListener;
import org.jivesoftware.smackx.filetransfer.FileTransferManager;
import org.jivesoftware.smackx.filetransfer.FileTransferRequest;
import org.jivesoftware.smackx.filetransfer.IncomingFileTransfer;
import org.jivesoftware.smackx.filetransfer.OutgoingFileTransfer;
import org.jivesoftware.smackx.filetransfer.FileTransfer.Status;
import org.jivesoftware.smackx.packet.VCard;
import org.jivesoftware.smackx.provider.DiscoverInfoProvider;
import org.jivesoftware.smackx.provider.DiscoverItemsProvider;

import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.app.Activity;
import android.app.Dialog;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.BitmapFactory;
import android.util.Base64;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;
import ar.com.daidalos.afiledialog.FileChooserDialog;

public class TestPanel extends Activity {
	//Server Configuration Parameters
    public static final String HOST = "192.168.1.103";
    public static final int PORT = 5222;
    public static final String SERVICE = "192.168.1.103";
    
    //Test User Configurations
    public static  String USERNAME = "receiver@"+HOST ;
    public static  String PASSWORD ="12345";
    
    private static Connection connection;
    
    //Maximum retry in XMPP requests
    public static final int MAX_RETRY = 5;
    private int retry_times=0;
	private String filenameWithPath="/storage/sdcard0/Pictures/avatar1.png";
    
    public static int new_avatar_index = 1;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_test_panel);
		
		XMPPConnection.DEBUG_ENABLED = true;
        ConnectionConfiguration connConfig = new ConnectionConfiguration(HOST, PORT, SERVICE);
        connection = new XMPPConnection(connConfig);
        
        ProviderManager.getInstance().addIQProvider("query","http://jabber.org/protocol/bytestreams", new BytestreamsProvider());
        ProviderManager.getInstance().addIQProvider("query","http://jabber.org/protocol/disco#items", new DiscoverItemsProvider());
        ProviderManager.getInstance().addIQProvider("query","http://jabber.org/protocol/disco#info", new DiscoverInfoProvider());
        
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.test_panel, menu);
		return true;
	}
	
	//Create Connection to XMPP Server
	public void makeConnection(View view)
	{
		
		if(connection.isConnected())
		{
        	//Log.e("TestPanel",  "[makeConnection] Already Connected to "+ connection.getHost());
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" Already Connected to "+connection.getHost());
			return;
		}
		
        // Create a connection
        ConnectionConfiguration connConfig = new ConnectionConfiguration(HOST, PORT, SERVICE);
        XMPPConnection.DEBUG_ENABLED = true;
        connection = new XMPPConnection(connConfig);
        
        try {
            connection.connect();
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText("Connected to "+connection.getHost());
            //Log.i("TestPanel",  "[makeConnection] Connected to "+connection.getHost());
        } catch (XMPPException ex) {
            Log.e("TestPanel",  "[makeConnection] Failed to connect to "+ connection.getHost());
            Log.e("TestPanel", ex.toString());
            
            retry_times++;
            if(retry_times>MAX_RETRY)
            {
            	Log.e("TestPanel",  "[makeConnection] max retries reached to connect "+ connection.getHost());
                TextView appmessage = (TextView) findViewById(R.id.test_status_message);
                appmessage.setText("Failed to connect to "+connection.getHost());
                retry_times=0;
            }
            else
            	makeConnection(view);
            	
        }
	}

	//Login to XMPP Server
	public void loginAccount(View view)
	{
		if(!connection.isConnected())
		{
        	Log.e("TestPanel",  "[loginAccount] first try connect to Server: "+ connection.getHost());
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText("First try connect to Server: "+connection.getHost());
			return;
		}
		
        try {
        	connection.login(USERNAME,PASSWORD);
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText("Logined As "+USERNAME+" to "+connection.getHost());
            //Log.i("TestPanel",  "[makeConnection] Connected to "+connection.getHost());
        } catch (XMPPException ex) {
            Log.e("TestPanel",  "[loginAccount] Failed to login to "+ connection.getHost());
            Log.e("TestPanel", ex.toString());
            
            retry_times++;
            if(retry_times>MAX_RETRY)
            {
            	Log.e("TestPanel",  "[loginAccount] max retries reached to connect "+ connection.getHost());
                TextView appmessage = (TextView) findViewById(R.id.test_status_message);
                appmessage.setText("Failed to login to "+connection.getHost());
                
                EditText logview = (EditText) findViewById(R.id.logbox);
                logview.setText(ex.toString());
                retry_times=0;
                fileTransferListener();
            }
            else
            	loginAccount(view);
            	
        }
        
	}
	
	//Login to XMPP Server
	public void logoutAccount(View view)
	{
		if (connection!=null && connection.isConnected()) {
			connection.disconnect();
	        
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText("Disconnected From "+connection.getHost());
		}
		else
		{
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText("Already Disconnected From "+connection.getHost());
		}
        
	}
	
	
	
	public void selectFile(View view)
	{
		
		 FileChooserDialog dialog = new FileChooserDialog(this);
		 
	     dialog.setFilter(".*jpg|.*png|.*gif|.*JPG|.*PNG|.*GIF");
	     dialog.show();
		 
	     dialog.addListener(new FileChooserDialog.OnFileSelectedListener() {
	         public void onFileSelected(Dialog source, File file) {
	             source.hide();
	             Toast toast = Toast.makeText(source.getContext(), getResources().getText(R.string.selected_file_message)+": " + file.getName(), Toast.LENGTH_LONG);
	             toast.show();
	             
	             filenameWithPath=file.getAbsolutePath();
	             
	             //set image preview source
	    		 File imgFile = new  File(filenameWithPath);
	    		 if(imgFile.exists())
	    		 {
	    		     ImageView myImage = (ImageView) findViewById(R.id.avatar_preview);
	    		     myImage.setImageURI(Uri.fromFile(imgFile));
	    		     
	    		     Button btn_confirm=(Button) findViewById(R.id.confirm_avatar_btn);
	    		     btn_confirm.setEnabled(true);
	    		     
	    			 TextView textView = (TextView) findViewById(R.id.update_avatar_result_text);
	    			 textView.setText(getResources().getText(R.string.selected_file_message)+": " + file.getName());
	    		 }
	    		 else
	    		 {
	    		     Button btn_confirm=(Button) findViewById(R.id.confirm_avatar_btn);
	    		     btn_confirm.setEnabled(false);
	    			 TextView textView = (TextView) findViewById(R.id.update_avatar_result_text);
	    			 textView.setText(getResources().getText(R.string.msg_no_valid_file_selected));
	    		 }
	         }
			@Override
			public void onFileSelected(Dialog source, File folder, String name) {
				// TODO Auto-generated method stub
				
			}
	     });
		
	     dialog.show();
	}
	
	
	
	public void sendFile(View view)
	{
		FileTransferManager manager = new FileTransferManager(connection);
			OutgoingFileTransfer transfer = manager.createOutgoingFileTransfer("receiver@"+HOST+"/Smack");
			
			File file = new File(filenameWithPath);
			try {
			   transfer.sendFile(file, file.getName());
			} catch (XMPPException e) {
				 Log.i("FILE TRANSFER", "FAILED Message: " + transfer.getError());
			}
			Log.i("File Transfer", "Start Sending . . .");
			while(!transfer.isDone()) {
			   if(transfer.getStatus().equals(Status.error)) {
			      Log.i("FILE TRANSFER", "ERROR!!! " + transfer.getError());
			   } else if (transfer.getStatus().equals(Status.cancelled)
			                    || transfer.getStatus().equals(Status.refused)) {
				   Log.i("FILE TRANSFER", "Cancelled!!! " + transfer.getError());
			   }
			   try {
			      Thread.sleep(1000L);
			   } catch (InterruptedException e) {
				   Log.i("FILE TRANSFER", " Failed in Thread: " + transfer.getError());
			   }
			}
			if(transfer.getStatus().equals(Status.refused) || transfer.getStatus().equals(Status.error)
			 || transfer.getStatus().equals(Status.cancelled)){
				 Log.i("FILE TRANSFER", "refused cancelled error " + transfer.getError());
			} else {
   			 TextView textView = (TextView) findViewById(R.id.update_avatar_result_text);
   			 textView.setText("File Transferd: " + file.getName());
			}
			
	}
	
	public void fileTransferListener()
	{
		FileTransferManager manager = new FileTransferManager(connection);
			manager.addFileTransferListener(new FileTransferListener() {
			   public void fileTransferRequest(final FileTransferRequest request) {
			      new Thread(){
			         @Override
			         public void run() {
		    			 TextView textView = (TextView) findViewById(R.id.update_avatar_result_text);
		    			 textView.setText("Somthing Recieved");
			            IncomingFileTransfer transfer = request.accept();
			            File mf = Environment.getExternalStorageDirectory();
			            File file = new File("/storage/sdcard0/Pictures/recieved.png");
			            try{
			                transfer.recieveFile(file);
			                while(!transfer.isDone()) {
			                   try{
			                      Thread.sleep(1000L);
			                   }catch (Exception e) {
			                      Log.i("Error Recieving", e.getMessage());
			                   }
			                   if(transfer.getStatus().equals(Status.error)) {
			                      Log.i("ERROR!!! ", transfer.getError() + "");
			                   }
			                   if(transfer.getException() != null) {
			                      //transfer.getException().printStackTrace();
			                	   Log.i("Error Recieving", transfer.getException().toString());
			                   }
			                }
			             }catch (Exception e) {
			            	 Log.i("Error Recieving2", e.getMessage());
			            }
			         };
			       }.start();
			    }
			 });
	}
	
	public void changeAvatar(View view)
	{
		if(!connection.isConnected())
		{
        	Log.e("TestPanel",  "[changeAvatar] first try connect to Server: "+ connection.getHost());
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText("First try connect to Server: "+connection.getHost());
			return;
		}
		
		if( new_avatar_index == 1)
		{
			loadVCard(connection,"/storage/sdcard0/Pictures/avatar1.png");
			new_avatar_index=2;
		}
		else
		{
			loadVCard(connection,"/storage/sdcard0/Pictures/avatar2.png");
			new_avatar_index=1;
		}
		
	}
	
	
	// Load vCard of logged in user
	private void loadVCard(Connection conn, String avatar_address) {
	    VCard vcard = new VCard();
	    try {
			vcard.load(conn);
		} catch (XMPPException e) {
        	Log.e("TestPanel",  "[loadVCard] error loading vcard");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error loading vcard ");
		}
	    
	    //resize any avatar image to fit 80*80pixcels
	    Bitmap b = BitmapFactory.decodeFile(avatar_address);
	    Bitmap b2 = Bitmap.createScaledBitmap(b, 80, 80, false);
	    
	    /*
	    //file descriptor for avatar image 
	    File file = new File(avatar_address);
        // Reading a Image file from file system
        FileInputStream imageInFile = null;
		try {
			imageInFile = new FileInputStream(file);
		} catch (FileNotFoundException e1) {
        	Log.e("TestPanel",  "[loadVCard] error reading avatar file");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error reading file ");
		}
		
        byte imageData[] = new byte[(int) file.length()];
        try {
			imageInFile.read(imageData);
		} catch (IOException e1) {
        	Log.e("TestPanel",  "[loadVCard] error reading avatar file");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error reading image ");
		}
		*/
	    //convert resized bitmap to byte array
	    byte imageData[] = getByteArray(b2);
        // Converting Image byte array into Base64 String
        String imageDataString = encodeImage(imageData);
	    
	    vcard.setField("PHOTO", "<TYPE>image/jpg</TYPE><BINVAL>" + imageDataString + "</BINVAL>", true);
	    
	    if(imageDataString!="")
	    {
	    	EditText logbox = (EditText) findViewById(R.id.logbox);
	    	logbox.setText(imageDataString);
	    }
	    else
	    {
        	Log.e("TestPanel",  "[loadVCard] error loading avatar from url");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error loading avatar from url");
            return;
	    }
	    
	    try {
			vcard.save(conn);
			
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText("VCard saved, Avatar Changeed Successfully");
            
		} catch (XMPPException e) {
        	Log.e("TestPanel",  "[loadVCard] error saving vcard");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error saving vcard");
		}
	}

	
    public static String encodeImage(byte[] imageByteArray) {
        return Base64.encodeToString(imageByteArray,Base64.DEFAULT);
    }
    
    //convert Bitmap to ByteArray
    public byte[] getByteArray(Bitmap bitmap) {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        bitmap.compress(CompressFormat.PNG, 0, bos);
        return bos.toByteArray();
    }
	/*
	// Load vCard of logged in user
	private void loadVCard(XMPPConnection conn, String avatar_address) {
	    VCard vcard = new VCard();
	    try {
			vcard.load(conn);
		} catch (XMPPException e) {
        	Log.e("TestPanel",  "[loadVCard] error loading vcard");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error loading vcard ");
		}

	    //vcard.setFirstName("TestUser");
	    URL avatarUrl = null;
        byte[] bytes = new byte[0];
        
		try {
			avatarUrl = new URL(avatar_address);
	        try {
	            bytes = VCard.getBytes(avatarUrl);
	        }
	        catch (IOException e) {
	        	Log.e("TestPanel",  "[loadVCard] error loading avatar from url");
	            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
	            appmessage.setText(" error loading avatar from url");
	            return;
	        }

		} catch (MalformedURLException e) {
        	Log.e("TestPanel",  "[loadVCard] error loading avatar from url");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error loading avatar from url");
		}
	    vcard.setAvatar(bytes,"image/jpeg");
	    
	    if(bytes!=null)
	    {
	    	EditText logbox = (EditText) findViewById(R.id.logbox);
	    	logbox.setText(StringUtils.encodeBase64(bytes));
	    }
	    else
	    {
        	Log.e("TestPanel",  "[loadVCard] error loading avatar from url");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error loading avatar from url");
            return;
	    }
	    
	    try {
			vcard.save(conn);
			
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText("VCard saved, Avatar Changeed Successfully");
            
		} catch (XMPPException e) {
        	Log.e("TestPanel",  "[loadVCard] error saving vcard");
            TextView appmessage = (TextView) findViewById(R.id.test_status_message);
            appmessage.setText(" error saving vcard");
		}
	}
*/
	// Load vCard of give user
	// username: ex mylogin@chat.myservice.com
	/*
	private void loadVCard(XMPPConnection conn, String username) {
	    VCard vcard = new VCard();
	    try {
			vcard.load(conn, username);
		} catch (XMPPException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	    byte[] avatarBytes = vcard.getAvatar();
	    OutputStream out = null;
		try {
			out = new FileOutputStream(username + ".jpg");
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	    try {
			out.write(avatarBytes);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	    try {
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	*/
	
}
